<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Block\Click;

use Flat101\Base\Service\GetCurrentProductService as CurrentProduct;
use Flat101\Base\Service\GetCurrentCategoryService as CurrentCategory;
use Flat101\EnhancedEcommerce\Block\Datalayer;
use Flat101\EnhancedEcommerce\Model\DataLayer\Impressions\CatalogImpressionView;
use Flat101\EnhancedEcommerce\Model\DataLayer\Impressions\ProductRelatedView;
use Flat101\EnhancedEcommerce\Model\DataLayer\Impressions\SearchImpressionView;
use Flat101\EnhancedEcommerce\Api\DataLayerListInterface;
use Flat101\EnhancedEcommerce\Factory\DataLayerFactory;
use Flat101\EnhancedEcommerce\Helper\Config;
use Magento\Framework\View\Element\Template;
use Magento\Framework\App\Response\RedirectInterface;
use Magento\Framework\Event\Manager;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Framework\UrlInterface;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;

class Data extends Datalayer
{
    /** @var CatalogImpressionView */
    protected $catalogImpressionView;

    /** @var ProductRelatedView */
    protected $relatedImpressionView;

    /** @var SearchImpressionView */
    protected $searchImpressionView;

    /** @var CurrentCategory */
    protected $currentCategory;

    /** @var CollectionFactory */
    protected $collectionFactory;

    public function __construct(
        CurrentProduct $currentProduct,
        CurrentCategory $currentCategory,
        CatalogImpressionView $catalogImpressionView,
        ProductRelatedView $relatedImpressionView,
        SearchImpressionView $searchImpressionView,
        Template\Context $context,
        DataLayerListInterface $dataLayerList,
        DataLayerFactory $dataLayerFactory,
        SerializerInterface $serializer,
        UrlInterface $url,
        Config $config,
        RedirectInterface $redirect,
        Manager $eventManage,
        CollectionFactory $collectionFactory,
        array $data = []
    ) {
        parent::__construct($currentProduct, $context, $dataLayerList, $dataLayerFactory, $serializer, $url, $config, $redirect, $eventManage, $data);
        $this->catalogImpressionView = $catalogImpressionView;
        $this->relatedImpressionView = $relatedImpressionView;
        $this->searchImpressionView = $searchImpressionView;
        $this->serializer = $serializer;
        $this->config = $config;
        $this->currentCategory = $currentCategory;
        $this->collectionFactory = $collectionFactory;
    }

    public function getJsonClickData(): ?string
    {
        $impressionData = null;

        $currentFullAction = $this->getRequest()->getFullActionName();
        $plpPage = ['catalog_category_view','ambrand_index_index'];
        $pdpPage = ['catalog_product_view'];
        $searchPage = ['catalogsearch_result_index'];

        if (in_array($currentFullAction, $plpPage)){
            $impressionData = $this->catalogImpressionView->getLayer();
        }

        if (in_array($currentFullAction, $pdpPage)){
            $impressionData = $this->relatedImpressionView->getImpressionsData();
        }

        if (in_array($currentFullAction, $searchPage)){
            $impressionData = $this->searchImpressionView->getLayer();
        }

        if (is_null($impressionData))
            return $this->serializer->serialize('');

        $response = [];
        foreach ($impressionData as $impression) {
            $prodId = $impression->id;
            if ($this->config->getProductIdentifier() === 'sku') {
                $prodId = $this->getProdIdBySku($impression->id);
            }

            $response[$prodId] = [
                'list'      => $impression->list,
                'category'  => $impression->category,
                'position'  => $impression->position
            ];
        }
        return $this->serializer->serialize($response);
    }

    public function getAjaxUrl(): string
    {
        return $this->getUrl('flat101_enhancedecommerce/click/view');
    }

    protected function getProdIdBySku($sku): string
    {
        $collection = $this->collectionFactory->create()
            ->addAttributeToSelect('entity_id')
            ->addAttributeToFilter('sku', $sku);

        return $collection->getFirstItem()->getId();
    }
}
